from __future__ import annotations

import json
from enum import Enum

API_REFERENCE = "scalar-api-reference.js"


class SearchHotKey(Enum):
    A = "a"
    B = "b"
    C = "c"
    D = "d"
    E = "e"
    F = "f"
    G = "g"
    H = "h"
    I = "i"  # noqa: E741
    J = "j"
    K = "k"
    L = "l"
    M = "m"
    N = "n"
    O = "o"  # noqa: E741
    P = "p"
    Q = "q"
    R = "r"
    S = "s"
    T = "t"
    U = "u"
    V = "v"
    W = "w"
    X = "x"
    Y = "y"
    Z = "z"


class ThemeId(Enum):
    ALTERNATE = "alternate"
    DEFAULT = "default"
    MOON = "moon"
    PURPLE = "purple"
    SOLARIZED = "solarized"
    BLUE_PLANET = "bluePlanet"
    DEEP_SPACE = "deepSpace"
    SATURN = "saturn"
    KEPLER = "kepler"
    ELYSIAJS = "elysiajs"
    FASTIFY = "fastify"
    MARS = "mars"
    LASERWAVE = "laserwave"
    NONE = "none"


class Integration(Enum):
    ADONISJS = "adonisjs"
    DOCUSAURUS = "docusaurus"
    DOTNET = "dotnet"
    ELYSIAJS = "elysiajs"
    EXPRESS = "express"
    FASTAPI = "fastapi"
    FASTIFY = "fastify"
    GO = "go"
    HONO = "hono"
    HTML = "html"
    LARAVEL = "laravel"
    LITESTAR = "litestar"
    NESTJS = "nestjs"
    NEXTJS = "nextjs"
    NITRO = "nitro"
    NUXT = "nuxt"
    PLATFORMATIC = "platformatic"
    REACT = "react"
    RUST = "rust"
    SVELTE = "svelte"
    VUE = "vue"


def dump(key: str, data: object) -> str:
    if not isinstance(data, bool) and not data:
        return ""
    return f"{key}: {json.dumps(data)},"


def get_scalar_html(  # noqa: PLR0913
    *,
    openapi_url: str,
    title: str,
    scalar_js_url: str = "https://cdn.jsdelivr.net/npm/@scalar/api-reference",
    scalar_proxy_url: str = "",
    scalar_favicon_url: str = "https://docs.scalar.com/favicon.svg",
    slug: str | None = None,
    authentication: dict | None = None,
    base_server_url: str = "",
    hide_client_button: bool = False,
    proxy_url: str = "",
    search_hot_key: SearchHotKey = SearchHotKey.K,
    servers: list[dict[str, str]] | None = None,
    show_sidebar: bool = True,
    theme: ThemeId = ThemeId.DEFAULT,
    integration: Integration | None = None,
    persist_auth: bool = False,
) -> str:
    """
    Generates HTML for Scalar API Reference based on OpenAPI specification.

    https://github.com/scalar/scalar/blob/main/packages/types/src/api-reference/api-reference-configuration.ts

    Args:
        openapi_url: URL to an OpenAPI/Swagger document
        title: The title of the OpenAPI document.
        scalar_js_url: URL to Scalar JavaScript library
        scalar_proxy_url: Scalar proxy URL
        scalar_favicon_url: Favicon URL
        slug: The slug of the OpenAPI document used in the URL. If none is passed, the title will be used.
        authentication: Prefill authentication
        base_server_url: Base URL for the API server
        hide_client_button: Whether to hide the client button
        proxy_url: URL to a request proxy for the API client
        search_hot_key: Key used with CTRL/CMD to open the search modal (defaults to 'k' e.g. CMD+k)
        servers: List of OpenAPI server objects
        show_sidebar: Whether to show the sidebar
        theme:  A string to use one of the color presets
        integration: Integration type identifier
        persist_auth: Whether to persist auth to local storage

    Returns:
        HTML document displaying the Scalar API Reference
    """
    servers = servers or []
    authentication = authentication or {}

    return f"""<!doctype html>
<html>
  <head>
    <title>{title}</title>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1" />
    {f'<link rel="shortcut icon" href="{scalar_favicon_url}">' if scalar_favicon_url else ""}
  </head>
  <body>
    <script
      id="api-reference"
      data-url="{openapi_url}"
      {f'data-proxy-url="{scalar_proxy_url}"' if scalar_proxy_url else ""}
    >
    </script>

    <script>
      var configuration = {{
          {dump("slug", slug)}
          {dump("authentication", authentication)}
          {dump("baseServerUrl", base_server_url)}
          {dump("hideClientButton", hide_client_button)}
          {dump("proxyUrl", proxy_url)}
          {dump("searchHotKey", search_hot_key.value)}
          {dump("servers", servers)}
          {dump("showSidebar", show_sidebar)}
          {dump("theme", theme.value)}
          {dump("_integration", integration.value if integration else None)}
          {dump("persistAuth", persist_auth)}
      }}

      document.getElementById('api-reference').dataset.configuration =
        JSON.stringify(configuration)
    </script>

    <script src="{scalar_js_url}"></script>
  </body>
</html>"""



from enum import Enum


class TryItCredentialPolicyOptions(Enum):
    OMIT = "omit"
    INCLUDE = "include"
    SAME_ORIGIN = "same-origin"


class LayoutOptions(Enum):
    SIDEBAR = "sidebar"
    RESPONSIVE = "responsive"
    STACKED = "stacked"


class RouterOptions(Enum):
    HISTORY = "history"
    HASH = "hash"
    MEMORY = "memory"
    STATIC = "static"


def get_stoplight_elements_html(  # noqa: PLR0913 many arguments in function definition
    *,
    openapi_url: str,
    title: str,
    stoplight_elements_js_url: str = "https://cdn.jsdelivr.net/npm/@stoplight/elements/web-components.min.js",
    stoplight_elements_css_url: str = "https://cdn.jsdelivr.net/npm/@stoplight/elements/styles.min.css",
    stoplight_elements_favicon_url: str = "https://docs.stoplight.io/favicons/favicon.ico",
    api_description_document: str = "",
    base_path: str = "",
    hide_internal: bool = False,
    hide_try_it: bool = False,
    hide_export: bool = False,
    try_it_cors_proxy: str = "",
    try_it_credential_policy: TryItCredentialPolicyOptions = TryItCredentialPolicyOptions.OMIT,
    layout: LayoutOptions = LayoutOptions.SIDEBAR,
    logo: str = "",
    router: RouterOptions = RouterOptions.HASH,
) -> str:
    """
    Generate an HTML document that embeds the Stoplight Elements API Explorer.

    https://docs.stoplight.io/docs/elements/b074dc47b2826-elements-configuration-options

    Args:
        openapi_url: OpenAPI document URL, supporting `http://`, `https://`, and documents containing `$ref` to other http(s) documents.
        title: The title of the HTML document.
        stoplight_elements_js_url: URL to the Stoplight Elements JavaScript file.
        stoplight_elements_css_url: URL to the Stoplight Elements CSS file.
        stoplight_elements_favicon_url: URL to the Stoplight Elements favicon.
        api_description_document: OpenAPI document, provided as YAML string, JSON string, or JavaScript object.
        base_path: Helps when using `router: 'history'` but docs are in a subdirectory like `https://example.com/docs/api`.
        hide_internal: Pass `True` to filter out any content which has been marked as internal with `x-internal`.
        hide_try_it: Pass `True` to hide the [Try It feature](https://docs.stoplight.io/docs/platform/ZG9jOjM2OTM3Mjky-try-it).
        hide_export: Pass `True` to hide the Export button on overview section of the documentation.
        try_it_cors_proxy: Pass the URL of a CORS proxy used to send requests to the Try It feature. The provided URL is pre-pended to the URL of an actual request.
        try_it_credential_policy: Use to fetch the credential policy for the Try It feature. Options are: `omit` (default), `include`, and `same-origin`.
        layout: There are three layouts for Elements:
          - `sidebar` - (default) Three-column design with a sidebar that can be resized.
          - `responsive` - Like `sidebar`, except at small screen sizes it collapses the sidebar into a drawer that can be toggled open.
          - `stacked` - Everything in a single column, making integrations with existing websites that have their own sidebar or other columns already.
        logo: URL to an image that displays as a small square logo next to the title, above the table of contents.
        router:  Determines how navigation should work:
          - `history` - uses the HTML5 history API to keep the UI in sync with the URL.
          - `hash` - (default) uses the hash portion of the URL to keep the UI in sync with the URL.
          - `memory` - keeps the history of your "URL" in memory (doesn't read or write to the address bar).
          - `static` - renders using the StaticRouter which can help render pages on the server.

    Returns:
        The HTML document as a string.
    """

    return f"""
<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <title>{title}</title>

    {f'<link rel="shortcut icon" href="{stoplight_elements_favicon_url}">' if stoplight_elements_favicon_url else ""}
    <script src="{stoplight_elements_js_url}"></script>
    <link rel="stylesheet" href="{stoplight_elements_css_url}">
  </head>
<body>
  <elements-api
    {f'apiDescriptionUrl="{openapi_url}"' if openapi_url else ""}
    {f'apiDescriptionDocument="{api_description_document}"' if api_description_document else ""}
    {f'basePath="{base_path}"' if base_path else ""}
    {'hideInternal="true"' if hide_internal else ""}
    {'hideTryIt="true"' if hide_try_it else ""}
    {'hideExport="true"' if hide_export else ""}
    {f'tryItCorsProxy="{try_it_cors_proxy}"' if try_it_cors_proxy else ""}
    tryItCredentialPolicy="{try_it_credential_policy.value}"
    layout="{layout.value}"
    {f'logo="{logo}"' if logo else ""}
    router="{router.value}"
  />
</body>
</html>
"""



from .__version__ import __version__
from .elements import (
    LayoutOptions,
    RouterOptions,
    TryItCredentialPolicyOptions,
    get_stoplight_elements_html,
)

# Literal strings for file names
WEB_COMPONENTS = "web-components.min.js"
"web-components.min.js"
STYLES = "styles.min.css"
"styles.min.css"
FAVICON = "favicon.ico"
"favicon.ico"

__all__ = [
    "FAVICON",
    "STYLES",
    "WEB_COMPONENTS",
    "LayoutOptions",
    "RouterOptions",
    "TryItCredentialPolicyOptions",
    "__version__",
    "get_stoplight_elements_html",
]



from importlib.resources import files as _files
from typing import Final

_MODULE = "stopments.static"

favicon_content: Final[bytes] = _files(_MODULE).joinpath("favicon.ico").read_bytes()
"""
Favicon content from https://docs.stoplight.io/favicons/favicon.ico

Content-Type: image/x-icon

Example
-------

```python
from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from stopments.embed import favicon_content

app = FastAPI()

@app.get("/favicon.ico", include_in_schema=False)
async def favicon_ico():
    return HTMLResponse(content=favicon_content, media_type="image/x-icon")
```
"""


css_content: Final[bytes] = _files(_MODULE).joinpath("styles.min.css").read_bytes()
"""
CSS content from https://cdn.jsdelivr.net/npm/@stoplight/elements/styles.min.css

Content-Type: text/css; charset=utf-8

Example
-------

```python
from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from stopments.embed import css_content

app = FastAPI()

@app.get("/static/styles.min.css", include_in_schema=False)
async def styles_css():
    return HTMLResponse(content=css_content, media_type="text/css; charset=utf-8")
```
"""

js_content: Final[bytes] = (
    _files(_MODULE).joinpath("web-components.min.js").read_bytes()
)
"""
JavaScript content from https://cdn.jsdelivr.net/npm/@stoplight/elements/web-components.min.js

Content-Type: application/javascript; charset=utf-8

Example
-------

```python
from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from stopments.embed import js_content

app = FastAPI()

@app.get("/static/web-components.min.js", include_in_schema=False)
async def web_components_js():
    return HTMLResponse(
        content=js_content, media_type="application/javascript; charset=utf-8"
    )
```
"""

scalar_api_reference_js_content: Final[bytes] = (
    _files(_MODULE).joinpath("scalar-api-reference.js").read_bytes()
)
"""
JavaScript content from https://cdn.jsdelivr.net/npm/@scalar/api-reference
Content-Type: application/javascript; charset=utf-8

Example
-------

```python
from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from stopments.embed import scalar_api_reference_js_content

app = FastAPI()

@app.get("/static/scalar-api-reference.js", include_in_schema=False)
async def scalar_api_reference_js():
    return HTMLResponse(
        content=scalar_api_reference_js_content, media_type="application/javascript; charset=utf-8"
    )
```
"""

del _files, _MODULE

__all__ = [
    "css_content",
    "favicon_content",
    "js_content",
    "scalar_api_reference_js_content",
]
